// Copyright (c) 2012-2013, IGN France.
// Copyright (c) 2012-2024, Oslandia.
// Copyright (c) 2024-2025, SFCGAL team.
// SPDX-License-Identifier: LGPL-2.0-or-later

#ifndef SFCGAL_UBLAS_H_
#define SFCGAL_UBLAS_H_

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>

namespace SFCGAL {
namespace detail {
namespace ublas = boost::numeric::ublas;
} // namespace detail
} // namespace SFCGAL

#endif
